@@ -1,11 +1,13 @@
 """Allows CloudWanderer to store resources in memory."""
 import logging
 from typing import Callable, Iterator, List
+
 import boto3
-from .base_connector import BaseStorageConnector
-from ..aws_urn import AwsUrn
+
 from ..cloud_wanderer_resource import CloudWandererResource
+from ..urn import URN
 from ..utils import standardise_data_types
+from .base_connector import BaseStorageConnector
 
 logger = logging.getLogger(__name__)
 
@@ -23,30 +25,30 @@ class MemoryStorageConnector(BaseStorageConnector):
 
     """
 
-    def __init__(self, *args, **kwargs) -> None:
+    def __init__(self) -> None:
         self._data = {}
 
     def init(self) -> None:
         """Do nothing. Dummy method to fulfil interface requirements."""
 
-    def read_resource(self, urn: AwsUrn) -> CloudWandererResource:
+    def read_resource(self, urn: URN) -> CloudWandererResource:
         try:
             return memory_item_to_resource(urn, self._data[str(urn)], loader=self.read_resource)
         except KeyError:
             return None
 
-    def read_resources(self, **kwargs) -> Iterator['CloudWandererResource']:
+    def read_resources(self, **kwargs) -> Iterator["CloudWandererResource"]:
         for urn_str, items in self._data.items():
-            urn = AwsUrn.from_string(urn_str)
-            if kwargs.get('urn') is not None:
-                if urn == kwargs['urn']:
+            urn = URN.from_string(urn_str)
+            if kwargs.get("urn") is not None:
+                if urn == kwargs["urn"]:
                     yield memory_item_to_resource(urn, loader=self.read_resource)
                 continue
             if self._urn_matches_kwargs(urn, **kwargs):
                 yield memory_item_to_resource(urn, loader=self.read_resource)
 
-    def _urn_matches_kwargs(self, urn: AwsUrn, **kwargs) -> bool:
-        filter_items = ('account_id', 'region', 'service', 'resource_type')
+    def _urn_matches_kwargs(self, urn: URN, **kwargs) -> bool:
+        filter_items = ("account_id", "region", "service", "resource_type")
         for item in filter_items:
             if kwargs.get(item) is not None and getattr(urn, item) != kwargs[item]:
                 return False
@@ -58,29 +60,29 @@ class MemoryStorageConnector(BaseStorageConnector):
             for item_type, item in items.items():
                 yield {
                     **{
-                        'urn': urn_str,
-                        'attr': item_type,
+                        "urn": urn_str,
+                        "attr": item_type,
                     },
-                    **item
+                    **item,
                 }
 
     def write_resource(self, resource: CloudWandererResource) -> None:
         self._data[str(resource.urn)] = self._data.get(str(resource.urn), {})
-        self._data[str(resource.urn)]['BaseResource'] = standardise_data_types(
-            resource.cloudwanderer_metadata.resource_data)
+        self._data[str(resource.urn)]["BaseResource"] = standardise_data_types(
+            resource.cloudwanderer_metadata.resource_data
+        )
         for secondary_attribute in resource.cloudwanderer_metadata.secondary_attributes:
             self._write_secondary_attribute(
-                urn=resource.urn,
-                attribute_type=secondary_attribute.name,
-                secondary_attribute=secondary_attribute
+                urn=resource.urn, attribute_type=secondary_attribute.name, secondary_attribute=secondary_attribute
             )
 
     def _write_secondary_attribute(
-            self, urn: AwsUrn, attribute_type: str, secondary_attribute: boto3.resources.base.ServiceResource) -> None:
+        self, urn: URN, attribute_type: str, secondary_attribute: boto3.resources.base.ServiceResource
+    ) -> None:
         """Write the specified resource attribute to storage.
 
         Arguments:
-            urn (AwsUrn): The resource whose attribute to write.
+            urn (URN): The resource whose attribute to write.
             attribute_type (str): The type of the resource attribute to write (usually the boto3 client method name)
             secondary_attribute (boto3.resources.base.ServiceResource): The resource attribute to write to storage.
 
@@ -88,18 +90,18 @@ class MemoryStorageConnector(BaseStorageConnector):
         self._data[str(urn)] = self._data.get(str(urn), {})
         self._data[str(urn)][attribute_type] = secondary_attribute
 
-    def delete_resource(self, urn: AwsUrn) -> None:
+    def delete_resource(self, urn: URN) -> None:
         try:
             del self._data[str(urn)]
         except KeyError:
             pass
 
     def delete_resource_of_type_in_account_region(
-            self, service: str, resource_type: str, account_id: str,
-            region: str, urns_to_keep: List[AwsUrn] = None) -> None:
+        self, service: str, resource_type: str, account_id: str, region: str, urns_to_keep: List[URN] = None
+    ) -> None:
         urns_to_delete = []
         for urn_str, items in self._data.items():
-            urn = AwsUrn.from_string(urn_str)
+            urn = URN.from_string(urn_str)
             if urn.service != service:
                 continue
             if urn.account_id != account_id:
@@ -114,22 +116,25 @@ class MemoryStorageConnector(BaseStorageConnector):
         for urn in urns_to_delete:
             del self._data[str(urn)]
 
+    def __repr__(self) -> str:
+        """Return an instantiable string representation of this object."""
+        return f"{self.__class__.__name__}()"
 
-def memory_item_to_resource(urn: AwsUrn, items: dict = None, loader: Callable = None) -> CloudWandererResource:
+    def __str__(self) -> str:
+        """Return a string representation of this object."""
+        return f"<{self.__class__.__name__}>"
+
+
+def memory_item_to_resource(urn: URN, items: dict = None, loader: Callable = None) -> CloudWandererResource:
     """Convert a resource and its attributes to a CloudWandererResource.
 
     Arguments:
-        urn (AwsUrn): The URN of the resource.
+        urn (URN): The URN of the resource.
         items (dict): The dictionary of items stored under this URN. (Secondary Attributs, BaseResource)
         loader (Callable): The method which can be used to fulfil the :meth:`CloudWandererResource.load`
 
     """
     items = items or {}
-    attributes = [attribute for item_type, attribute in items.items() if item_type != 'BaseResource']
-    base_resource = next(iter(resource for item_type, resource in items.items() if item_type == 'BaseResource'), {})
-    return CloudWandererResource(
-        urn=urn,
-        resource_data=base_resource,
-        secondary_attributes=attributes,
-        loader=loader
-    )
+    attributes = [attribute for item_type, attribute in items.items() if item_type != "BaseResource"]
+    base_resource = next(iter(resource for item_type, resource in items.items() if item_type == "BaseResource"), {})
+    return CloudWandererResource(urn=urn, resource_data=base_resource, secondary_attributes=attributes, loader=loader)
